/* Copyright (c) 2012 Google Inc.
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

package com.nhnent.iap.sample;

import android.app.Activity;
import android.app.AlertDialog;
import android.content.DialogInterface;
import android.os.Bundle;
import android.util.Log;
import android.view.View;
import android.widget.ArrayAdapter;
import android.widget.EditText;
import android.widget.TextView;

import com.nhnent.iap.google.R;
import com.nhnent.iap.sample.service.IAPServiceProvider;
import com.nhnent.iap.sample.service.PurchaseItem;
import com.nhnent.iap.sample.service.QueryItem;
import com.nhnent.mobill.api.core.InAppConfigurationManager;
import com.nhnent.mobill.util.ApplicationUtils;
import com.nhnent.mobill.util.ObjectUtils;
import com.toast.android.iap.InAppPurchases;
import com.toast.android.iap.exception.InAppPurchaseException;

import java.util.ArrayList;

public class MainActivity extends Activity implements MainEventListener {
    public static final String TAG = "IAP TEST";
    private boolean isLogin = false;

    /**
     * Configuration activity life cycle.
     */

    @Override
    public void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        setContentView(R.layout.activity_main);

        /**
         * Enable Debug Mode IAP SDK, Available to show Logcat
         */
        InAppPurchases.InAppPurchase.setDebugMode(true);

        IAPServiceProvider.initialize(this);

        showAppEnvironment();
    }


    /******************************************************************
     * Receives processing results for the IAPServiceProvider method. *
     ******************************************************************/

    @Override
    public void onInAppPurchaseException(InAppPurchaseException exception) {
        alert("errorCode=" + exception.getErrorCode() + "\nmessage=" + exception.getMessage());
        setWaitScreen(false);
    }

    /**
     * Call after IAPServiceProvider.purchase()
     */
    @Override
    public void afterPurchase(PurchaseItem purchaseItem) {
        setWaitScreen(false);
        alert(purchaseItem.getItemSeq() + getString(R.string.sample_purchase_complete));
    }

    /**
     * Call after IAPServiceProvider.queryItems()
     */
    @Override
    public void afterQueryItems(ArrayList<QueryItem> queryItems) {
        setWaitScreen(false);
        if (queryItems.size() == 0) {
            alert(R.string.sample_there_are_no_items);
        } else {
            alertList(R.string.sample_query_item_title, queryItems);
        }
    }

    /**
     * Call after IAPServiceProvider.queryTCConsoleItems()
     */
    @Override
    public void afterQueryTcConsoleItems(ArrayList<QueryItem> queryItems) {
        setWaitScreen(false);

        if (queryItems.size() == 0) {
            alert(R.string.sample_there_are_no_items);
        } else {
            alertList(R.string.sample_query_tc_console_item_title, queryItems);
        }
    }

    /**
     * Call after IAPServiceProvider.queryIncompletePurchases()
     */
    @Override
    public void afterQueryIncompletePurchases(ArrayList<PurchaseItem> purchaseItems) {
        setWaitScreen(false);

        if (purchaseItems.size() == 0) {
            alert(R.string.sample_there_are_no_items);
        } else {
            alertList(R.string.sample_select_consume_item, purchaseItems);
        }
    }

    /**
     * Call after IAPServiceProvider.processesIncompletePurchases()
     */
    @Override
    public void afterProcessesIncompletePurchases(String message) {
        setWaitScreen(false);
        alert(message);
    }

    /**
     * Call after IAPServiceProvider.requestConsume()
     */
    @Override
    public void afterConsume(String message) {
        setWaitScreen(false);
        alert(message);
    }


    /**
     * User clicked button event
     */
    public void onClickedInit(View view) {
        String userId = ((EditText) findViewById(R.id.register_user_et)).getText().toString();

        if (ObjectUtils.isEmpty(userId)) {
            userId = getString(R.string.sample_default_user_id);
        }

        this.isLogin = true;

        // TODO: Register user identifier
        IAPServiceProvider.registerUserId(userId);

        alert(getString(R.string.sample_init_complete) + userId);
    }

    public void onClickedPurchase(View view) {
        if (!this.isLogin) {
            alert(R.string.sample_please_login);
            return;
        }

        String itemId = ((EditText) findViewById(R.id.item_id_et)).getText().toString();
        if (ObjectUtils.isEmpty(itemId)) {
            alert(R.string.sample_insert_item_id);
            return;
        }

        // launch the gas purchase UI flow.
        // We will be notified of completion via mPurchaseFinishedListener
        setWaitScreen(true);
        Log.d(TAG, "launching purchase flow for item.");

        // TODO: request purchase.
        IAPServiceProvider.purchase(this, Long.valueOf(itemId));
    }

    public void onClickedQueryItems(View view) {
        setWaitScreen(true);

        // TODO: query items.
        IAPServiceProvider.queryItems(this);
    }

    public void onClickedQueryTCConsoleItems(View view) {
        setWaitScreen(true);

        // TODO : query Toast cloud console items
        IAPServiceProvider.queryTCConsoleItems(this);
    }

    public void onClickedQueryIncompletePurchases(View view) {
        if (!this.isLogin) {
            alert(R.string.sample_please_login);
            return;
        }

        setWaitScreen(true);

        // TODO : query incomplete purchase
        IAPServiceProvider.queryIncompletePurchases(this);
    }

    public void onClickedProcessesIncompletePurchases(View view) {
        if (!this.isLogin) {
            alert(R.string.sample_please_login);
            return;
        }

        setWaitScreen(true);

        // TODO : processes incomplete purchases
        IAPServiceProvider.processesIncompletePurchases(this);
    }

    private void onClickedListItem(Object item) {
        if (item instanceof QueryItem) {
            setTextViewItemId(((QueryItem) item).getItemSeq());
        } else if (item instanceof PurchaseItem) {
            PurchaseItem purchaseItem = (PurchaseItem) item;

            // TODO: Query purchase list (Needs consume in server side)
            IAPServiceProvider.requestConsume(
                    purchaseItem.getPaymentSeq(),
                    purchaseItem.getPurchaseToken(),
                    purchaseItem.getItemSeq());
        }
    }

    /**
     * Set market detail
     */
    private void showAppEnvironment() {
        Bundle bundle = ApplicationUtils.getAndroidManifestMetaData(this);

        long appId = 0;
        String market = "";
        String server = "";

        if (InAppConfigurationManager.getInstance().getMarket() != null) {
            appId = InAppConfigurationManager.getInstance().getAppId();
            market = InAppConfigurationManager.getInstance().getMarket().getMarketId();
        } else if (bundle != null && bundle.get("com.toast.iap.config.appId") != null) {
            appId = (Integer) bundle.get("com.toast.iap.config.appId");
            market = bundle.getString("com.toast.iap.config.market");
        }
        Log.d(TAG, "appId: " + appId);
        Log.d(TAG, "market: " + market);

        // Real 로 고정되어 있음
        if (InAppConfigurationManager.getInstance().getServer() != null) {
            // Local, Alpha, Beta, Real
            server = InAppConfigurationManager.getInstance().getServer().name();
        } else if (bundle != null) {
            server = bundle.getString("com.toast.iap.config.server");
        }

        StringBuilder appInfo = new StringBuilder();
        appInfo.append("SdkVersion=");
        appInfo.append(InAppPurchases.SDK_VERSION);
        appInfo.append(", AppId=");
        appInfo.append(appId);

        StringBuilder buildType = new StringBuilder();
        buildType.append("Market=");
        buildType.append(market);
        buildType.append(", BuildType=");
        buildType.append(server == null ? "REAL" : server);

        ((TextView) findViewById(R.id.sample_app_info_tv)).setText(appInfo.toString());
        ((TextView) findViewById(R.id.sample_build_type_tv)).setText(buildType.toString());
    }

    /**
     * Set item id of EditText.
     *
     * @param itemId purchase item id
     */
    private void setTextViewItemId(Long itemId) {
        ((EditText) findViewById(R.id.item_id_et)).setText(String.valueOf(itemId));
    }

    /**
     * Enables or disables the "please wait" screen.
     *
     * @param set enable, disable
     */
    private void setWaitScreen(boolean set) {
        findViewById(R.id.screen_main).setVisibility(set ? View.GONE : View.VISIBLE);
        findViewById(R.id.screen_wait).setVisibility(set ? View.VISIBLE : View.GONE);
    }

    /**
     * Show alert dialog.
     *
     * @param resId String refers to resource ID
     */
    private void alert(int resId) {
        alert(getString(resId));
    }

    /**
     * Show alert dialog.
     *
     * @param message body message
     */
    private void alert(String message) {
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);
        dialogBuilder.setMessage(message);
        dialogBuilder.setNeutralButton("OK", null);
        Log.d(TAG, "Showing alert dialog: " + message);
        dialogBuilder.create().show();
    }

    /**
     * Show alert in list form.
     *
     * @param resId String refers to resource ID
     * @param items QueryItems or PurchaseItems
     */
    private void alertList(int resId, final ArrayList<?> items) {
        alertList(getString(resId), items);
    }

    /**
     * Show alert in list form.
     *
     * @param title title message
     * @param items QueryItems or PurchaseItems
     */
    private void alertList(String title, final ArrayList<?> items) {
        AlertDialog.Builder dialogBuilder = new AlertDialog.Builder(this);
        dialogBuilder.setTitle(title);

        ArrayAdapter<?> adapter = new ArrayAdapter<>(this, android.R.layout.select_dialog_singlechoice, items);
        dialogBuilder.setAdapter(adapter,
                new DialogInterface.OnClickListener() {
                    public void onClick(DialogInterface dialog, int id) {
                        onClickedListItem(items.get(id));
                    }
                });

        dialogBuilder.setNegativeButton(R.string.com_toast_iap_btn_cancel, null);
        dialogBuilder.show();
    }
}
