package com.nhnent.iap.sample.service;

import android.app.Activity;
import android.os.Message;
import android.util.Log;

import com.nhnent.iap.sample.MainEventListener;
import com.nhnent.mobill.api.core.InAppConfigurationManager;
import com.nhnent.mobill.api.core.InAppServer;
import com.nhnent.mobill.net.AbstractRequest.Builder;
import com.nhnent.mobill.net.AsyncRequestRunner;
import com.nhnent.mobill.net.HttpMethod;
import com.nhnent.mobill.net.RequestRunner;
import com.nhnent.mobill.net.ResponseListener;
import com.toast.android.iap.InAppPurchase;
import com.toast.android.iap.InAppPurchase.PurchaseListCallback;
import com.toast.android.iap.InAppPurchases;
import com.toast.android.iap.exception.InAppPurchaseException;

import org.json.JSONArray;
import org.json.JSONObject;

import java.util.ArrayList;

public class IAPServiceProvider {
    public static final String TAG = "[IAPServiceProvider]";

    private static IAPEventHandler mEventHandler;

    public static void initialize(MainEventListener listener) {
        registerListener(listener);
    }

    private static final void registerListener(MainEventListener listener) {
        mEventHandler = new IAPEventHandler(listener);
    }

    public static void registerUserId(String userId) {
        /**
         * 샘플 어플리케이션에서는 유저 등록을 디바이스ID를 통해 진행 하지만, 실제 사용자의 어플리케이션에서는
         * 로그인 이후에 고유한 유저 ID를 통하여 유저를 등록 해야 합니다.
         */
        InAppPurchases.InAppPurchase.registerUserId(userId);
    }

    public static void purchase(final Activity activity, long itemId) {
        /**
         * Web Console에 등록한 아이템 정보를 통해 결제 요청을 합니다.
         */
        InAppPurchases.InAppPurchase.requestPurchase(activity, itemId, new InAppPurchase.PurchaseCallback() {

            @Override
            public void onCallback(JSONObject result, InAppPurchaseException exception) {
                if (exception != null) {
                    // TODO : 에러 발생시에 대한 처리를 정의 합니다.
                    mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_ERROR_ACTION, exception));
                    return;
                }

                /**
                 * TODO
                 *  - 결제가 완료 되면 결제내역 정보가 전달 됩니다. 결제내역을 바탕으로 어플리케이션 서버에서 consume 후에 아이템 지급 처리를 하시면 됩니다.
                 *    샘플 어플리케이션에서는 클라이언트에서 진행 하지만, 애플리케이션의 서버에서 consume API 을 통해 검증 후에 아이템 지급하는 것을 강력하게 권장합니다.
                 *  @paymentSeq - 결제 고유 번호
                 *  @itemSeq - 아이템 번호
                 *  @purchaseToken - 결제 검증을 위한 토큰
                 *  @currency - 통화단위
                 *  @price - 가격
                 *
                 *  Response
                 *  {
                 *	   "paymentSeq": string,
                 *	   "purchaseToken": string,
                 *	   "itemSeq": long,
                 *     "currency": string,
                 *     "price": float
                 *	}
                 */

                /**
                 * Consume API
                 * [URL] 	  https://api-iap.cloud.toast.com/inapp/v2/consume/{paymentSeq}/items/{itemSeq}
                 * [Method]   POST
                 * [RequestBody]
                 * {
                 *	  "purchaseToken":string
                 * }
                 */


                mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_HANDLE_AFTER_PURCHASE, PurchaseItem.build(result)));
            }
        });

    }

    public static void queryIncompletePurchases(final Activity activity) {
        /**
         * 결제 내역중 IAP서버로 부터 Consume 되지 않은 내역을 조회 합니다. (Consume은 어플리케이션 서버에서 아이템 지급전 검증을 위한 단계 입니다. HTTP API Reference 참조)
         * 어플리케이션 실행시 마다 결제 내역을 조회하여 누락된 아이템 지급을 계속해서 진행 할 수 있습니다.
         */
        InAppPurchases.InAppPurchase.queryPurchases(activity, new PurchaseListCallback() {

            @Override
            public void onCallback(JSONArray result, InAppPurchaseException exception) {
                if (exception != null) {
                    // TODO : 에러 발생시에 대한 처리를 정의 합니다.
                    mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_ERROR_ACTION, exception));
                    return;
                }

                /**
                 * TODO
                 * 미소비 결제내역 조회
                 * - 아이템 지급 처리 중에 어플리케이션이 예상치 않은 문제로 중단 될수 있습니다. 미소비 결제내역 조회는 어플리케이션 실행시에 호출하여 지급되지 않은 아이템을 지급하는데에 사용됩니다.
                 *   샘플 어플리케이션에서는 클라이언트에서 진행 하지만, 애플리케이션의 서버에서 consume API 을 통해 검증 후에 아이템 지급하는 것을 강력하게 권장합니다.
                 * Examples)				 *
                 * [{
                 *	"paymentSeq": "2014082510002163",
                 *	"itemSeq": 1000208,
                 *	"purchaseToken": "8nkx3SzHKlI74vmgQLzHExmlS/0DSt4JDs2UMyg1/EY8oC6Q8qkuw5VBo7GNrBYLNUy656GCAh7h9e1BtXeoB-AB",
                 *  "currency": "KRW",
                 *  "price": 1000.0
                 *    },
                 *  {
                 *	"paymentSeq": "2014082510002164",
                 *	"itemSeq": 1000209,
                 *	"purchaseToken": "8nkx3SzATKlI74vmgQLzHExmlS/0DSt4JDs2UMyg1/EY8oC6Q8qkuw5VBo7GNrBYLNUy656GCAh7h9e1BtXeoBaAC",
                 *  "currency": "KRW",
                 *  "price": 1000.0
                 *    }]
                 */

                Log.d(TAG, result.toString());

                ArrayList<PurchaseItem> purchaseItems = new ArrayList<>();

                for (int i = 0; i < result.length(); i++) {
                    purchaseItems.add(PurchaseItem.build(result.optJSONObject(i)));
                }

                mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_HANDLE_AFTER_QUERY_INCOMPLETE_PURCHASES, purchaseItems));
            }
        });
    }

    public static void queryItems(final Activity activity) {
        /**
         * 구매 가능한 모든 상품 목록을 조회합니다.
         */
        InAppPurchases.InAppPurchase.queryItems(activity, new InAppPurchase.ItemListCallback() {
            @Override
            public void onCallback(JSONArray result, InAppPurchaseException exception) {
                if (exception != null) {
                    // TODO : 에러 발생시에 대한 처리를 정의 합니다.
                    mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_ERROR_ACTION, exception));
                    return;
                }

                /**
                 * 상품내역 조회
                 * Examples)
                 * [{
                 * 	"itemSeq" : 1000208,
                 * 	"itemName" : "Test item 01",
                 *	"marketItemId": "item01",
                 *  "price": 1000,
                 *  "currency": "KRW"
                 *    },
                 *  {
                 *  "itemSeq" : 1000209,
                 *  "itemName" : "Test item 02",
                 *	"marketItemId": "item02",
                 *  "price": 7.99,
                 *  "currency": "USD"
                 *    }]
                 */

                Log.d(TAG, result.toString());

                ArrayList<QueryItem> queryItems = new ArrayList<>();

                for (int i = 0; i < result.length(); i++) {
                    queryItems.add(QueryItem.build(result.optJSONObject(i)));
                }

                mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_HANDLE_AFTER_QUERY_ITEMS, queryItems));
            }
        });
    }

    public static void queryTCConsoleItems(final Activity activity) {
        /**
         * Toast Cloud Console에 등록된 모든 상품 목록을 조회합니다.
         */
        InAppPurchases.InAppPurchase.queryTCConsoleItems(activity, new InAppPurchase.ItemListCallback() {
            @Override
            public void onCallback(JSONArray result, InAppPurchaseException exception) {
                if (exception != null) {
                    // TODO : 에러 발생시에 대한 처리를 정의 합니다.
                    mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_ERROR_ACTION, exception));
                    return;
                }

                /**
                 * 토스트클라우드 콘솔 내 상품내역 조회
                 * Examples)
                 * [{
                 * 	"itemSeq" : 1000208,
                 * 	"itemName" : "Test item 01",
                 *	"marketItemId": "item01",
                 *  "usingStatus" : "USE",
                 *  "regYmdt" : "2017-01-04 14:17:30",
                 *  "appName" : "Test app",1
                 *  "marketId" : "GG"
                 *  },
                 *  {
                 *  "itemSeq" : 1000209,
                 *  "itemName" : "Test item 02",
                 *	"marketItemId": "item02",
                 *  "usingStatus" : "STOP",
                 *  "regYmdt" : "2017-01-04 14:17:30",
                 *  "appName" : "Test app",
                 *  "marketId" : "GG"
                 *  }]
                 */

                Log.d(TAG, result.toString());

                ArrayList<QueryItem> queryItems = new ArrayList<>();

                for (int i = 0; i < result.length(); i++) {
                    queryItems.add(QueryItem.build(result.optJSONObject(i)));
                }

                mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_HANDLE_AFTER_QUERY_TC_CONSOLE_ITEMS, queryItems));
            }
        });
    }

    public static void processesIncompletePurchases(final Activity activity) {
        /**
         * 결제 이후에 필요한 처리가 실패했던, 미완료된 구매건들을 찾아 재처리한다.
         */
        InAppPurchases.InAppPurchase.processesIncompletePurchases(activity, new InAppPurchase.IncompletePurchasesCallback() {
            @Override
            public void onCallback(JSONObject result, InAppPurchaseException exception) {
                if (exception != null) {
                    // TODO : 에러 발생시에 대한 처리를 정의 합니다.
                    mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_ERROR_ACTION, exception));
                    return;
                }

                /**
                 * TODO
                 *  @paymentSeq - 결제 고유 번호
                 *  @itemSeq - 아이템 번호
                 *  @purchaseToken - 결제 검증을 위한 토큰
                 *  @currency - 통화단위
                 *  @price - 가격
                 *
                 *  Response
                 *  {
                 *	   "successList": [
                 *	   		{
                 *	   			"paymentSeq" : string,
                 *	   			"purchaseToken" : string,
                 *	   			"itemSeq" : long,
                 *	   			"marketItemId"	: string,
                 *	   			"currency" : string,
                 *	   			"price" : double
                 *	   		},
                 *	   		{
                 *	   			"paymentSeq" : string,
                 *	   			"purchaseToken" : string,
                 *	   			"itemSeq" : long,
                 *	   		    "marketItemId"	: string,
                 *	   			"currency" : string,
                 *	   			"price" : double
                 *	   		}
                 *	   ],
                 *	   "failList": [
                 *	   		{
                 *	   			"paymentSeq" : string,
                 *	   			"purchaseToken" : null,
                 *	   			"itemSeq" : long,
                 *	   			"marketItemId"	: string,
                 *	   			"currency" : string,
                 *	   			"price" : double
                 *	   		}
                 *	   ]
                 *	}
                 */

                Log.d(TAG, result.toString());

                JSONArray successList = result.optJSONArray("successList");
                JSONArray failList = result.optJSONArray("failList");

                String successResultString = "Success : " + successList.length() + "\n";
                String failResultString = "Fail : " + failList.length() + "\n";

                for (int i = 0; i < successList.length(); i++) {
                    JSONObject payment = successList.optJSONObject(i);
                    PurchaseItem purchaseItem = PurchaseItem.build(payment);
                    successResultString += purchaseItem.getMarketItemId() + "(" + purchaseItem.getItemSeq() + ")\n";
                }

                for (int i = 0; i < failList.length(); i++) {
                    JSONObject payment = failList.optJSONObject(i);
                    PurchaseItem purchaseItem = PurchaseItem.build(payment);
                    failResultString += purchaseItem.getMarketItemId() + "(" + purchaseItem.getItemSeq() + ")\n";
                }

                mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_HANDLE_AFTER_PROCESS_INCOMPLETE_PURCHASES, successResultString + "\n" + failResultString));
            }
        });
    }

    /**
     * Call Payment Consume API Example
     * - IAP서버에 유효한 결제건인지 검증하는 단계입니다.
     * - Consume API는 애플리케이션의 서버에서 아이템지급전에 호출 하는 것을 권장 합니다.
     */

    public static void requestConsume(String paymentSeq, String purchaseToken, final Long itemSeq) {
        String consumeApiUrl = "https://api-iap.cloud.toast.com/inapp/v3/consume/" + paymentSeq + "/items/" + itemSeq;
        if (InAppConfigurationManager.getInstance().getServer() == InAppServer.BETA) {
            consumeApiUrl = "https://beta-api-iap.cloud.toast.com/inapp/v3/consume/" + paymentSeq + "/items/" + itemSeq;
        }

        Builder builder = new Builder(consumeApiUrl).method(HttpMethod.POST)
                .addParameter("purchaseToken", purchaseToken)
                .enableJsonBody(true);

        RequestRunner<String> runner = new AsyncRequestRunner(builder);
        runner.call(new ResponseListener<String>() {

            @Override
            public void onException(Exception exception) {
                Log.e(TAG, exception.getMessage(), exception);
            }

            @Override
            public void onComplete(String result) {
                Log.i(TAG, result);
                mEventHandler.sendMessage(Message.obtain(mEventHandler, IAPEventHandler.IAP_HANDLE_AFTER_CONSUME, itemSeq + " 지급 완료."));
            }
        });
    }

    public static void registerAppId(long appId) {
        /**
         * 사용할 ToastCloud Console의 appId를 등록합니다.
         * 별도 호출하지 않는 경우, AndroidManifest 파일에 등록된 appId를 사용합니다.
         */
        InAppPurchases.InAppPurchase.registerAppId(appId);
    }

    public static void registerMarketId(String marketId) {
        /**
         * 사용할 ToastCloud Console의 marketId를 등록합니다.
         * 별도 호출하지 않는 경우, AndroidManifest 파일에 등록된 marketId를 사용합니다.
         */
        InAppPurchases.InAppPurchase.registerMarketId(marketId);
    }

    public static void registerServerPhase(InAppServer serverPhase) {
        /**
         * 알맞은 서버 Phase를 설정합니다.
         * 별도 호출하지 않은 경우, AndroidManifest 파일에 등록된 값을 사용합니다.
         */
        InAppPurchases.InAppPurchase.registerServerPhase(serverPhase);
    }

}